// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0-or-later

package negotiation

import (
	"net/http"
)

// Negotiation represents the HTTP negotiator.
type Negotiation struct {
	header http.Header
}

// New creates an instance of Negotiation.
func New(header http.Header) *Negotiation {
	return &Negotiation{header}
}

func (n *Negotiation) ContentType(acceptance ...string) string {
	parser := newHeaderParser(n.header, false)
	return parser.selectOffer(acceptance, parser.parse(headerContentType))
}

// Type returns the most preferred content type from the HTTP Accept header.
// If nothing accepted, then empty string is returned.
func (n *Negotiation) Type(offers ...string) (bestOffer string) {
	parser := newHeaderParser(n.header, true)
	return parser.selectOffer(offers, parser.parse(headerAccept))
}

// Language returns the most preferred language from the HTTP Accept-Language
// header. If nothing accepted, then empty string is returned.
func (n *Negotiation) Language(offers ...string) (bestOffer string) {
	parser := newHeaderParser(n.header, false)
	return parser.selectOffer(offers, parser.parse(headerAcceptLanguage))
}

// Encoding returns the most preferred encoding from the HTTP Accept-Encoding
// header. If nothing accepted, then empty string is returned.
func (n *Negotiation) Encoding(offers ...string) (bestOffer string) {
	parser := newHeaderParser(n.header, false)
	return parser.selectOffer(offers, parser.parse(headerAcceptEncoding))
}

// Charset returns the most preferred charset from the HTTP Accept-Charset
// header. If nothing accepted, then empty string is returned.
func (n *Negotiation) Charset(offers ...string) (bestOffer string) {
	parser := newHeaderParser(n.header, false)
	return parser.selectOffer(offers, parser.parse(headerAcceptCharset))
}
